@@ -23,6 +23,8 @@
 # include <unistd.h>
 #endif
 
+#include <errno.h>
+
 #include	"tclInt.h"	/* Internal definitions for Tcl. */
 
 #include "tcl.h"
@@ -34,6 +36,8 @@
 #include "exp_command.h"
 #include "exp_log.h"
 
+static int		ExpBlockModeProc _ANSI_ARGS_((ClientData instanceData,
+			    int mode));
 static int		ExpCloseProc _ANSI_ARGS_((ClientData instanceData,
 			    Tcl_Interp *interp));
 static int		ExpInputProc _ANSI_ARGS_((ClientData instanceData,
@@ -52,8 +56,14 @@ static int		ExpGetHandleProc _ANSI_ARGS_((ClientData instanceData,
 
 Tcl_ChannelType expChannelType = {
     "exp",				/* Type name. */
-    /* Expect channels are always blocking */
-    NULL,				/* Set blocking/nonblocking mode.*/
+
+    /* Tcl_ChannelType was redefined in 8.3.2 but Tcl does not
+       advertise its patch level in a useful way so for simplicity,
+       assume 8.3 is a modern 8.3, i.e. 8.3.2
+    */
+
+#if (TCL_MAJOR_VERSION > 8) || ((TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION >= 3))
+    TCL_CHANNEL_VERSION_2,
     ExpCloseProc,			/* Close proc. */
     ExpInputProc,			/* Input proc. */
     ExpOutputProc,			/* Output proc. */
@@ -63,6 +73,22 @@ Tcl_ChannelType expChannelType = {
     ExpWatchProc,			/* Initialize notifier. */
     ExpGetHandleProc,			/* Get OS handles out of channel. */
     NULL,				/* Close2 proc */
+    ExpBlockModeProc,			/* Set blocking/nonblocking mode.*/
+    NULL,				/* Flush proc. */
+    NULL,				/* Handle channel event proc. */
+#else
+    /* Expect channels are always non-blocking */
+    ExpBlockModeProc,			/* Set blocking/nonblocking mode.*/
+    ExpCloseProc,			/* Close proc. */
+    ExpInputProc,			/* Input proc. */
+    ExpOutputProc,			/* Output proc. */
+    NULL,				/* Seek proc. */
+    NULL,				/* Set option proc. */
+    NULL,				/* Get option proc. */
+    ExpWatchProc,			/* Initialize notifier. */
+    ExpGetHandleProc,			/* Get OS handles out of channel. */
+    NULL,				/* Close2 proc */
+#endif
 };
 
 typedef struct ThreadSpecificData {
@@ -79,6 +105,61 @@ typedef struct ThreadSpecificData {
 
 static Tcl_ThreadDataKey dataKey;
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * ExpBlockModeProc --
+ *
+ *	Helper procedure to set blocking and nonblocking modes on a
+ *	file based channel. Invoked by generic IO level code.
+ *
+ * Results:
+ *	0 if successful, errno when failed.
+ *
+ * Side effects:
+ *	Sets the device into blocking or non-blocking mode.
+ *
+ *----------------------------------------------------------------------
+ */
+
+	/* ARGSUSED */
+static int
+ExpBlockModeProc(instanceData, mode)
+    ClientData instanceData;		/* Exp state. */
+    int mode;				/* The mode to set. Can be one of
+					 * TCL_MODE_BLOCKING or
+					 * TCL_MODE_NONBLOCKING. */
+{
+    ExpState *esPtr = (ExpState *) instanceData;
+    int curStatus;
+    /*printf("ExpBlockModeProc(%d)\n",mode);
+      printf("fdin = %d\n",esPtr->fdin);*/
+
+#ifndef USE_FIONBIO
+    curStatus = fcntl(esPtr->fdin, F_GETFL);
+    /*printf("curStatus = %d\n",curStatus);*/
+    if (mode == TCL_MODE_BLOCKING) {
+	curStatus &= (~(O_NONBLOCK));
+    } else {
+	curStatus |= O_NONBLOCK;
+    }
+    /*printf("new curStatus %d\n",curStatus);*/
+    if (fcntl(esPtr->fdin, F_SETFL, curStatus) < 0) {
+	return errno;
+    }
+    curStatus = fcntl(esPtr->fdin, F_GETFL);
+#else /* USE_FIONBIO */
+    if (mode == TCL_MODE_BLOCKING) {
+	curStatus = 0;
+    } else {
+	curStatus = 1;
+    }
+    if (ioctl(esPtr->fdin, (int) FIONBIO, &curStatus) < 0) {
+	return errno;
+    }
+#endif /* !USE_FIONBIO */
+    return 0;
+}
 
 /*
  *----------------------------------------------------------------------
@@ -368,6 +449,22 @@ expSizeZero(esPtr)
     return (len == 0);
 }
 
+/* return 0 for success or negative for failure */
+int
+expWriteChars(esPtr,buffer,lenBytes)
+     ExpState *esPtr;
+     char *buffer;
+     int lenBytes;
+{
+  int rc;
+ retry:
+  rc = Tcl_WriteChars(esPtr->channel,buffer,lenBytes);
+  if ((rc == -1) && (errno == EAGAIN)) goto retry;
+
+  /* just return 0 rather than positive byte counts */
+  return ((rc > 0) ? 0 : rc);
+}
+
 void
 expStateFree(esPtr)
     ExpState *esPtr;
@@ -534,6 +631,7 @@ expCreateChannel(interp,fdin,fdout,pid)
     esPtr->echoed = 0;
     esPtr->rm_nulls = exp_default_rm_nulls;
     esPtr->parity = exp_default_parity;
+    esPtr->close_on_eof = exp_default_close_on_eof;
     esPtr->key = expect_key++;
     esPtr->force_read = FALSE;
     esPtr->fg_armed = FALSE;
